"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 06, 2023
Version: 1.0.0
Description: Knowledge base, module contains all frames of objects
"""


class Flat:
    """
    Class Flat represents a frame of flat

    Attributes
    ----------
    name: str
        name or id of flat
    address: str
        address of the flat
    floor: int
        floor where is the flat placed
    owner: str
        person who owns the flat
    area: int
        area in square meters
    flat_type: str
        type of flat (2+1, 3+1, 4+1, etc.)
    rooms: List[Room]
        rooms that the flat contains
    """

    def __init__(self, name, address, floor, owner, area, flat_type, rooms):
        """
        Constructor of class Flat
        """
        self.name = name
        self.address = address
        self.floor = floor
        self.owner = owner
        self.area = area
        self.flat_type = flat_type
        self.rooms = rooms

    def __str__(self):
        """
        Text representation of class
        """
        return f"Flat {self.name} is on address {self.address} with owner {self.owner}. Flat is on {self.floor} with " \
               f"flat type {self.flat_type} and area {self.area} square meters. "


class Room:
    """
    class Room represents frame of room

    Attributes
    ---------
    name: str
        name of the room
    is_in: str
        in which flat instance is the flat
    room_type: str
        type of room (living room, kitchen, etc.)
    area: int
        area of flat in square meters
    dimensions: str
        dimensions of the flat in meters. For example 3 x 4
    floor: str
        type of floor
    windows: List[]
        List of windows in room
    doors: List[]
        List of doors in room
    furniture: List[]
        List of furniture in room
    electrical_devices: List[]
        List of electrical devices in room
    """

    def __init__(self, name, is_in, room_type, area, dimensions, floor, windows, doors, furniture, electrical_devices):
        """
        Constructor of class Room
        """
        self.name = name
        self.is_in = is_in
        self.room_type = room_type
        self.area = area
        self.dimensions = dimensions
        self.floor = floor
        self.windows = windows
        self.doors = doors
        self.furniture = furniture
        self.electrical_devices = electrical_devices

    def __str__(self):
        """
        Text representation of class
        """
        return f"Room {self.name} is {self.room_type} with area {self.area} square meters. Floor is covered with {self.floor}. It " \
               f"has {str(len(self.windows))} windows and {str(len(self.doors))} doors."


class Window:
    """
    class Window represents frame of window

    Attributes
    ---------
    name: str
        name of the window
    is_in: str
        name of room what the window is in
    width: int
        width of window in millimeters
    height: int
        height of window in millimeters
    orientation: str
        orientation of window
    material: str
        material of the window
    color: str
        color of the window
    """
    def __init__(self, name, is_in, width, height, orientation, material, color):
        """
        Constructor of class Window
        """
        self.name = name
        self.is_in = is_in
        self.width = width
        self.height = height
        self.orientation = orientation
        self.material = material
        self.color = color

    def __str__(self):
        """
        Text representation of class
        """
        return f"Window {self.name} is in room {self.is_in} oriented to {self.orientation}. It is created of" \
               f" {self.material} with {self.color} color and dimensions {self.width}mm x {self.height}mm."


class Door:
    """
    class Door represents frame of door

    Attributes
    ---------
    name: str
        name of the door
    room1: str
        name of room what the door connect
    room2:
        name of room what the door connect
    height: int
        height of window in millimeters
    width: int
        width of window in millimeters
    material: str
        material of window
    color: str
        color of the window
    """
    def __init__(self, name, room1, room2, height, width, material, color):
        """
        Constructor of class Door
        """
        self.name = name
        self.room1 = room1
        self.room2 = room2
        self.height = height
        self.width = width
        self.material = material
        self.color = color

    def __str__(self):
        """
        Text representation of class
        """
        return f"Door {self.name} is in room {self.room1} leads to {self.room2}. It is created of" \
               f" {self.material} with {self.color} color and dimensions {self.width}mm x {self.height}mm."


class Furniture:
    """
    class Furniture represents frame of furniture

    Attributes
    ---------
    name: str
        name of the furniture
    is_in: str
        name of room what the furniture is in
    furniture_type: str
        type of furniture (Bed, Desk, etc.)
    position: str
        position in the room
    """
    def __init__(self, name, is_in, furniture_type, position):
        """
        Constructor of class Furniture
        """
        self.name = name
        self.is_in = is_in
        self.furniture_type = furniture_type
        self.position = position

    def __str__(self):
        """
        Text representation of class
        """
        return f"Furniture {self.name} is {self.furniture_type} in room {self.is_in} with position {self.position}."


class Electric_device:
    """
    class Electric_device represents frame of electric_device

    Attributes
    ---------
    name: str
        name of the device
    is_in: str
        name of room what the device is in
    device_type: str
        type of device (Computer, Television, etc.)
    brand: str
        manufacturer
    power: int
        power of the device
    position: str
        position in the room
    """
    def __init__(self, name, is_in, device_type, brand, power, position):
        """
        Constructor of class Electric devices
        """
        self.name = name
        self.is_in = is_in
        self.device_type = device_type
        self.brand = brand
        self.power = power
        self.position = position

    def __str__(self):
        """
        Text representation of class
        """
        return f"Electric device {self.name} is {self.device_type} in room {self.is_in} with position " \
               f"{self.position}. Manufacturer is {self.brand} with power {self.power}W."
