"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 06, 2023
Version: 1.0.0
Description: Communication module, this module contains all strings what can be print to the user
"""

author = "Lukáš Runt"

version = "1.0.0"

separator = "------------------------------------------------------"

welcome_text = f"Welcome to the expert system with database with flats.\nAutor: {author}\nVerze: {version}\n{separator}"

question = "What do you want to know?"

enter_input = "Enter your choice: "

basic_choices = "1) Print objects\n2) Where is object\n3) Which rooms connected by door\n4) Print information about " \
                "about\n5) For printing help\n6) Quit application"

farewell = "I hope that you enjoyed this program.\nHave a nice day!"

invalid_choice = "Invalid choice!"

help = "To control application enter a number (1-6) and its parameters.\nParameters for all types of commands:\n" \
       "1 <type_of_object> <where>\n" \
       "2 <name_of_instance>\n" \
       "3 <name_of_door>\n" \
       "4 <name_of_instance>\n" \
       "5\n" \
       "6\n" \
       "---------------------------------------------------------------------\n" \
       "<types_of_object> = All, Flat, Room, Window, Door, Furniture, Device\n" \
       "<where> = where we are looking for objects - name of instance Flat, Room or All"

instructions_to_continue = "Press ENTER to continue."

invalid_input = "The command is not valid. If you want to print help, enter 5."

invalid_frame = "The first parameter is not right!\nExpected one of these types: All, Flat, Room, Window, Door, Furniture, Device\n\nThere is also option, that you want to know flats in flat, flat in rooms or rooms in room."

invalid_where = "The second parameter is not right!\nExpected one of these types: All, <flat-name>, <room-name>"


def print_error_wrong_number_of_parameters(number, input):
    """
    Method prints that user input a wrong number of parameters
    :param number: number of choice
    :param input: user input.
    :return: error message.
    """
    return f"Wrong number of parameters. Expect: {number} <object> , but it was: {input}."


def print_error_object_not_exist(object):
    """
    Method prints that object what has user enter don't exit
    :param object: object what user has entered.
    :return: error message.
    """
    return f"Object {object} don't exist in database."


def print_error_help(input):
    """
    Method prints error when user probably wants to see a help
    :param input: user input.
    :return: error message.
    """
    return f"You probably want to print help, but you enter the invalid input: {input}.\n To print help you must enter 4"


def print_error_quit(input):
    """
    Method prints error when user probably wants to see a help
    :param input: user input
    :return: error message
    """
    return f"You probably want to quit, but you enter the invalid input: {input}.\n To quit you must enter 5"


def print_is_placed_in(instance, type):
    """
    Method print where is the instance placed in
    :param instance: instance what place we are looking for
    :param type: type of the instance
    :return: message where is the object located
    """
    if type == 0:
        return f"Flat {instance.name} has address {instance.address}."
    if type == 1:
        return f"Room {instance.name} is in flat {instance.is_in}."
    if type == 2:
        return f"Window {instance.name} is in room {instance.is_in} oriented to {instance.orientation}."
    if type == 3:
        return f"Door {instance.name} connects room {instance.room1} and room {instance.room2}."
    if type == 4:
        return f"Furniture {instance.name} is in room {instance.is_in} {instance.position}."
