"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 06, 2023
Version: 1.0.0
Description: Explanation module of expert system, getting and printing result to the user
"""

import KomunikacniModul
import BazeDat


def print_all(command):
    """
    method prints names of object
    It's ugly, but I am lazy to do it better
    :param command: the user's input command
    :return: 1 to indicate the command was processed, 0 there was an error
    """
    parsed = command.split()
    if len(parsed) != 3:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("1 <type_of_room>", command))
    else:
        if parsed[2] == "All":
            if parsed[1] == "All":
                print_all_names(BazeDat.flats)
                print_all_names(BazeDat.rooms)
                print_all_names(BazeDat.windows)
                print_all_names(BazeDat.doors)
                print_all_names(BazeDat.furniture)
                print_all_names(BazeDat.electric_devices)
            elif parsed[1] == "Flat":
                print_all_names(BazeDat.flats)
            elif parsed[1] == "Room":
                print_all_names(BazeDat.rooms)
            elif parsed[1] == "Door":
                print_all_names(BazeDat.doors)
            elif parsed[1] == "Window":
                print_all_names(BazeDat.windows)
            elif parsed[1] == "Furniture":
                print_all_names(BazeDat.furniture)
            elif parsed[1] == "Device":
                print_all_names(BazeDat.electric_devices)
            else:
                print(KomunikacniModul.invalid_frame)
                return 0
        elif get_flat(parsed[2]) is not None:
            flat = get_flat(parsed[2])
            rooms = flat.rooms
            if parsed[1] == "All":
                print_all_names(rooms)
                for room in rooms:
                    print_all_names(room.windows)
                    print_all_names(room.doors)
                    print_all_names(room.furniture)
                    print_all_names(room.electrical_devices)
            elif parsed[1] == "Room":
                print_all_names(rooms)
            elif parsed[1] == "Door":
                for room in rooms:
                    print_all_names(room.doors)
            elif parsed[1] == "Window":
                for room in rooms:
                    print_all_names(room.windows)
            elif parsed[1] == "Furniture":
                for room in rooms:
                    print_all_names(room.furniture)
            elif parsed[1] == "Device":
                for room in rooms:
                    print_all_names(room.electrical_devices)
            else:
                print(KomunikacniModul.invalid_frame)
                return 0
        elif get_room(parsed[2]) is not None:
            room = get_room(parsed[2])
            if parsed[1] == "All":
                print_all_names(room.windows)
                print_all_names(room.doors)
                print_all_names(room.furniture)
                print_all_names(room.electrical_devices)
            elif parsed[1] == "Door":
                print_all_names(room.doors)
            elif parsed[1] == "Window":
                print_all_names(room.windows)
            elif parsed[1] == "Furniture":
                print_all_names(room.furniture)
            elif parsed[1] == "Device":
                print_all_names(room.electrical_devices)
            else:
                print(KomunikacniModul.invalid_frame)
                return 0
        else:
            print(KomunikacniModul.invalid_where)
            return 0
    return 1


def where_is(command):
    """
    Method prints where is an object
    :param command: user command
    :return: 2 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("2", command))
        return 0
    elif not placed_in(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 2


def doors_connect(command):
    """
    Method prints what rooms the doors connects
    :param command: user command
    :return: 3 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("3", command))
    elif not what_door_connects(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 3


def print_information(command):
    """
    Method prints information about object
    :param command: user command
    :return: 4 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("4", command))
        return 0
    elif not exist(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 3


def print_help(command):
    """
    Method prints help to user
    :param command: user command
    :return: 5 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 1:
        print(KomunikacniModul.print_error_help(command))
        return 0
    print(KomunikacniModul.help)
    return 5


def exit(command):
    """
    Method to exit the application
    :param command: user command
    :return: 6 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 1:
        print(KomunikacniModul.print_error_quit(command))
        return 0
    return 6


# -------------------------------------------------------------------------
# --------------Helping methods--------------------------------------------
# -------------------------------------------------------------------------

def exist(name):
    """
    Method tests if there is an object in the database
    :param name: name of the object
    :return: True - Object exists, False - object do not exist
    """
    if exist_in_array(name, BazeDat.flats):
        return True
    if exist_in_array(name, BazeDat.rooms):
        return True
    if exist_in_array(name, BazeDat.windows):
        return True
    if exist_in_array(name, BazeDat.doors):
        return True
    if exist_in_array(name, BazeDat.furniture):
        return True
    if exist_in_array(name, BazeDat.electric_devices):
        return True
    return False


def exist_in_array(name, array):
    """
    Method tests if the object exists in an array
    :param name: name of the object
    :param array: name of the array,where we are looking for an object
    :return: True - object exists, False - object do not exist
    """
    for instance in array:
        if instance.name == name:
            print(instance)
            return True
    return False


def placed_in(name):
    """
    Method prints where is object placed
    :param name: name of the object
    :return: True - object exists, False - object do not exist
    """
    instance = get_object(name, BazeDat.flats)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 0))
        return True
    instance = get_object(name, BazeDat.rooms)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 1))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.windows)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 2))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.doors)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 3))
        placed_in(instance.room1)
        placed_in(instance.room2)
        return True
    instance = get_object(name, BazeDat.furniture)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 4))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.electric_devices)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 4))
        placed_in(instance.is_in)
        return True
    return False


def what_door_connects(door_name):
    """
    Method prints what rooms the doors connects
    :param door_name: name of the doors
    :return: True - doors exists, False - doors do not exist
    """
    instance = get_object(door_name, BazeDat.doors)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 3))
        return True
    return False


def get_object(name, array):
    """
    Method returns instance, if instance exists in array
    :param name: name of the object
    :param array: array where we are looking for the instance
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in array:
        if instance.name == name:
            return instance
    return None


def get_flat(name):
    """
    Method returns flat instance, if instance exists in array
    :param name: name of the flat
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in BazeDat.flats:
        if instance.name == name:
            return instance
    return None


def get_room(name):
    """
    Method returns room instance, if instance exists in array
    :param name: name of the room
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in BazeDat.rooms:
        if instance.name == name:
            return instance
    return None


def print_all_names(array):
    """
    Method prints all names in array
    :param array: array what will be printed
    """
    for instance in array:
        print(instance.name)


def get_all_where(array, string):
    """
    Method returns all objects in object
    :param array: in which array we are looking for an objects
    :param string: what object we are looking for
    :return: filered array
    """
    result = []
    for element in array:
        if element.is_in == string:
            result.append(element)
    return result
