"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 06, 2023
Version: 1.0.0
Description: Explanation module of expert system, getting and printing result to the user
"""

import KomunikacniModul
import BazeDat


def print_all():
    """
    Printing of all instances in database
    """
    print(KomunikacniModul.print_all)
    print(KomunikacniModul.print_flats_all)
    print_all_names(BazeDat.flats)
    print(KomunikacniModul.print_rooms_all)
    print_all_names(BazeDat.rooms)
    print(KomunikacniModul.print_window_all)
    print_all_names(BazeDat.windows)
    print(KomunikacniModul.print_door_all)
    print_all_names(BazeDat.doors)
    print(KomunikacniModul.print_furniture_all)
    print_all_names(BazeDat.furniture)
    print(KomunikacniModul.print_electric_devices_all)
    print_all_names(BazeDat.electric_devices)


def print_flats():
    """
    Printing of all instances of flat in database
    """
    print(KomunikacniModul.print_flats)
    print_all_names(BazeDat.flats)


def print_rooms():
    """
    Printing of all instances of room in database
    """
    print(KomunikacniModul.print_rooms)
    print_all_names(BazeDat.rooms)


def print_windows():
    """
    Printing of all instances of window in database
    """
    print(KomunikacniModul.print_window)
    print_all_names(BazeDat.windows)


def print_doors():
    """
    Printing of all instances of door in database
    """
    print(KomunikacniModul.print_door)
    print_all_names(BazeDat.doors)


def print_furniture():
    """
    Printing of all instances of furniture in database
    """
    print(KomunikacniModul.print_furniture)
    print_all_names(BazeDat.furniture)


def print_electric_devices():
    """
    Printing of all electric device in database
    """
    print(KomunikacniModul.print_electric_devices)
    print_all_names(BazeDat.electric_devices)


def where_is(command):
    """
    Method prints where is an object
    :param command: user command
    :return: 2 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("2", command))
        return 0
    elif not placed_in(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 2


def doors_connect(command):
    """
    Method prints what rooms the doors connects
    :param command: user command
    :return: 3 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("3", command))
    elif not what_door_connects(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 3


def print_information(command):
    """
    Method prints information about object
    :param command: user command
    :return: 4 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 2:
        print(KomunikacniModul.print_error_wrong_number_of_parameters("4", command))
        return 0
    elif not exist(parsed[1]):
        print(KomunikacniModul.print_error_object_not_exist(parsed[1]))
        return 0
    return 3


def print_help(command):
    """
    Method prints help to user
    :param command: user command
    :return: 5 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 1:
        print(KomunikacniModul.print_error_help(command))
        return 0
    print(KomunikacniModul.help)
    return 5


def exit(command):
    """
    Method to exit the application
    :param command: user command
    :return: 6 - success, 0 - there was an error
    """
    parsed = command.split()
    if len(parsed) != 1:
        print(KomunikacniModul.print_error_quit(command))
        return 0
    return 6


# -------------------------------------------------------------------------
# --------------Helping methods--------------------------------------------
# -------------------------------------------------------------------------

def exist(name):
    """
    Method tests if there is an object in the database
    :param name: name of the object
    :return: True - Object exists, False - object do not exist
    """
    if exist_in_array(name, BazeDat.flats):
        return True
    if exist_in_array(name, BazeDat.rooms):
        return True
    if exist_in_array(name, BazeDat.windows):
        return True
    if exist_in_array(name, BazeDat.doors):
        return True
    if exist_in_array(name, BazeDat.furniture):
        return True
    if exist_in_array(name, BazeDat.electric_devices):
        return True
    return False


def exist_in_array(name, array):
    """
    Method tests if the object exists in an array
    :param name: name of the object
    :param array: name of the array,where we are looking for an object
    :return: True - object exists, False - object do not exist
    """
    for instance in array:
        if instance.name == name:
            print(instance)
            return True
    return False


def placed_in(name):
    """
    Method prints where is object placed
    :param name: name of the object
    :return: True - object exists, False - object do not exist
    """
    instance = get_object(name, BazeDat.flats)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 0))
        return True
    instance = get_object(name, BazeDat.rooms)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 1))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.windows)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 2))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.doors)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 3))
        placed_in(instance.room1)
        placed_in(instance.room2)
        return True
    instance = get_object(name, BazeDat.furniture)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 4))
        placed_in(instance.is_in)
        return True
    instance = get_object(name, BazeDat.electric_devices)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 5))
        placed_in(instance.is_in)
        return True
    return False


def what_door_connects(door_name):
    """
    Method prints what rooms the doors connects
    :param door_name: name of the doors
    :return: True - doors exists, False - doors do not exist
    """
    instance = get_object(door_name, BazeDat.doors)
    if instance is not None:
        print(KomunikacniModul.print_is_placed_in(instance, 3))
        return True
    return False


def get_object(name, array):
    """
    Method returns instance, if instance exists in array
    :param name: name of the object
    :param array: array where we are looking for the instance
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in array:
        if instance.name == name:
            return instance
    return None


def get_flat(name):
    """
    Method returns flat instance, if instance exists in array
    :param name: name of the flat
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in BazeDat.flats:
        if instance.name == name:
            return instance
    return None


def get_room(name):
    """
    Method returns room instance, if instance exists in array
    :param name: name of the room
    :return: instance if instance was found, None - if wasn't found
    """
    for instance in BazeDat.rooms:
        if instance.name == name:
            return instance
    return None


def print_all_names(array):
    """
    Method prints all names in array
    :param array: array what will be printed
    """
    for instance in array:
        print(f"{instance.name} - {instance}")


def get_all_where(array, string):
    """
    Method returns all objects in object
    :param array: in which array we are looking for an objects
    :param string: what object we are looking for
    :return: filered array
    """
    result = []
    for element in array:
        if element.is_in == string:
            result.append(element)
    return result
